# This is the classic 1D Carbon box mode.
# This version has four boxes:
# (atmosphere,bosphere, ocean, deep ocean).
# Rodhe, Tellus (1979)
# Grey
# 2015-03-23

from math import *
import math
import matplotlib.pyplot as plt

PPM2GT = 2.123
CTOCO2 = 3.664


def biosphere_carbon_uptake():

    #Data from average fasir NDVI for 1996

    fasir=[0.97,0.94,0.98,1.12,1.38,1.86,2.43,2.39,1.78,1.35,1.16,1.03]

    total=0
    for i in range(12):
        total=total+fasir[i]

    for i in range(12):
        fasir[i]=fasir[i]/total

    return fasir

def box_model(initialAtmosCarbon, years, total_anthropogenic_emissions):        		

    # Step Time (dt)          
    dt=1.0

    years=56

    # Initial values (Gt) of Carbon reservoir stores
    atmosCarbonStoreStart =initialAtmosCarbon*PPM2GT   
    surfaceOceanCarbonStore = 1000.0 	
    terrestrialBiosphereCarbonStore = 3000.0   
    #deepOceanCarbonStore = 35000.0  	  
    atmosCarbonStore = [0 for i in range(years+1)]
    atmosCarbonStore[0]=atmosCarbonStoreStart

    # Flux coefficients
    atmosCarbonCoeff=0.15
    oceanDegassingCoeff = 1e-25
    oceanDegassingCoeffPow= 9.0    
    photosynthesisCoeff = 16.4 
    photosynthesisCoeffPow =0.2   
    respirationCoeff = 0.019                  
    #upwellingAndBiopumpCoeff = 0.00129               
    #downwellingCoeff = 0.0450              

    # Initial values of fluxes (Gt/yr)
    oceanCarbonUptakeFlux = atmosCarbonStore[0] *atmosCarbonCoeff                 
    oceanDegassingFlux =  oceanDegassingCoeff * math.pow(surfaceOceanCarbonStore, oceanDegassingCoeffPow)    
    photosynthesisFlux = photosynthesisCoeff  * math.pow(atmosCarbonStore[0],photosynthesisCoeffPow)    
    respirationFlux = respirationCoeff * terrestrialBiosphereCarbonStore                   
    #upwellingAndBiopumpFlux = upwellingAndBiopumpCoeff * deepOceanCarbonStore               
    #downwellingFlux = downwellingCoeff * surfaceOceanCarbonStore                

    for i in range(years):

        atmosCarbonStore[i+1] = atmosCarbonStore[i] + (respirationFlux + oceanDegassingFlux + total_anthropogenic_emissions[i] -  photosynthesisFlux - oceanCarbonUptakeFlux) * dt
        surfaceOceanCarbonStore = surfaceOceanCarbonStore + (oceanCarbonUptakeFlux - oceanDegassingFlux) * dt
        #surfaceOceanCarbonStore = surfaceOceanCarbonStore + (oceanCarbonUptakeFlux + upwellingAndBiopumpFlux - oceanDegassingFlux -downwellingFlux) * dt
        #deepOceanCarbonStore = deepOceanCarbonStore + (downwellingFlux - upwellingAndBiopumpFlux) * dt
        terrestrialBiosphereCarbonStore = terrestrialBiosphereCarbonStore + (photosynthesisFlux - respirationFlux) * dt
        oceanCarbonUptakeFlux = atmosCarbonStore[i] * 0.15             
        oceanDegassingFlux = 1e-25 * math.pow(surfaceOceanCarbonStore,9.0)          
        photosynthesisFlux = 16.4 * math.pow(atmosCarbonStore[i],0.2)           
        respirationFlux = 0.019 *terrestrialBiosphereCarbonStore                
        #upwellingAndBiopumpFlux = 0.00129 * deepOceanCarbonStore             
        #downwellingFlux = 0.0450 * surfaceOceanCarbonStore

        

     #Convert back to PPM.
    for i in range(years+1):
        atmosCarbonStore[i]=atmosCarbonStore[i]/PPM2GT

    return atmosCarbonStore


def box_model_monthly(initialAtmosCarbon, years, total_anthropogenic_emissions):        		

    # Step Time (dt)          
    dt=1.0/12
    years=56

    # Initial values (Gt) of Carbon reservoir stores
    atmosCarbonStoreStart =initialAtmosCarbon*PPM2GT   
    surfaceOceanCarbonStore = 1000.0 	
    terrestrialBiosphereCarbonStore = 3000.0   	  
    atmosCarbonStore = [0 for i in range(years*12+1)]
    atmosCarbonStore[0]=atmosCarbonStoreStart

    # Flux coefficients
    atmosCarbonCoeff=0.15
    oceanDegassingCoeff = 1e-25
    oceanDegassingCoeffPow= 9.0    
    photosynthesisCoeff = 16.4 
    photosynthesisCoeffPow =0.2   
    respirationCoeff = 0.019                  

    # Initial values of fluxes (Gt/yr)
    oceanCarbonUptakeFlux = atmosCarbonStore[0] *atmosCarbonCoeff             
    oceanDegassingFlux =  oceanDegassingCoeff * math.pow(surfaceOceanCarbonStore, oceanDegassingCoeffPow)  
    photosynthesisFlux = photosynthesisCoeff  * math.pow(atmosCarbonStore[0],photosynthesisCoeffPow) 
    respirationFlux = respirationCoeff * terrestrialBiosphereCarbonStore                 
     
    intra_annual_biosphere_uptake=biosphere_carbon_uptake()

    for i in range(years):
        for j in range(12):

            atmosCarbonStore[i*12+j+1] = atmosCarbonStore[i*12+j] + ( respirationFlux + oceanDegassingFlux + total_anthropogenic_emissions[i] -   photosynthesisFlux - oceanCarbonUptakeFlux) * dt
            surfaceOceanCarbonStore = surfaceOceanCarbonStore + (oceanCarbonUptakeFlux - oceanDegassingFlux) * dt
            terrestrialBiosphereCarbonStore = terrestrialBiosphereCarbonStore + (photosynthesisFlux - respirationFlux) * dt
            oceanCarbonUptakeFlux = atmosCarbonStore[i] * 0.15
            oceanDegassingFlux = 1e-25 * math.pow(surfaceOceanCarbonStore,9.0)      
            photosynthesisFlux = 16.4 * math.pow(atmosCarbonStore[i*12+j],0.2)         
            respirationFlux = 0.019 *terrestrialBiosphereCarbonStore               

     #Convert back to PPM.
    for i in range(years):
        for j in range(12):
            atmosCarbonStore[i*12+j]=atmosCarbonStore[i*12+j]/PPM2GT
    atmosCarbonStore[years*12]=atmosCarbonStore[years*12]/PPM2GT
    return atmosCarbonStore

def plot_carbon(co2):

# CO2 concentration in ppm between 1959 and 2014
# from NOAA Mauna Loa observatory record from:
# http://co2now.org/Current-CO2/CO2-Now/noaa-mauna-loa-co2-data.html */
    co2AtmosConcentrationInPPM =[
    315.97,316.91,317.64,318.45,318.99,319.62,320.04,321.38,322.16,323.04,
    324.62,325.68,326.32,327.45,329.68,330.18,331.08,332.05,333.78,335.41,
    336.78,338.68,340.10,341.44,343.03,344.58,346.04,347.39,349.16,351.56,
    353.07,354.35,355.57,356.38,357.07,358.82,360.80,362.59,363.71,366.65,
    368.33,369.52,371.13,373.22,375.77,377.49,379.80,381.90,383.76,385.59,
    387.37,389.85,391.63,393.82,396.48,398.55]

    plt.title("Atmospheric Carbon")
    plt.xlabel("Time") 
    plt.ylabel("Atmospheric carbon PPM") 
    #plt.xticks(rotation=90) 
    plt.plot(co2)
    plt.plot(co2AtmosConcentrationInPPM)

    fig = plt.gcf() 
    fig.subplots_adjust(bottom=0.2)
    fig.savefig("carbon.png", format='png')


def calc_total_anthropogenic_emissions():

    # data from http://www.globalcarbonproject.org/carbonbudget
    # 1959-2017
    
    fossil_fuel_emissions=[2.45, 2.57, 2.58, 2.69, 2.83, 2.99, 3.13, 3.29,
                                 3.39, 3.57, 3.78, 4.05, 4.21 ,4.37, 4.61,
                                 4.62, 4.59, 4.86, 5.01, 5.07, 5.35, 5.29,
                                 5.13, 5.08, 5.06, 5.24, 5.40, 5.57, 5.71,
                                 5.92, 6.05, 6.05, 6.12, 6.06, 6.05, 6.15,
                                 6.28, 6.42, 6.53, 6.55, 6.53, 6.70, 6.86,
                                 6.96, 7.33, 7.69, 7.98, 8.27, 8.43, 8.70,
                                 8.60, 9.02, 9.38, 9.53, 9.61, 9.69, 9.68, 9.74, 9.87]

    land_use_change_emissions=[1.81, 1.67, 1.61, 1.57, 1.51, 1.46,
                                             1.42, 1.38, 1.35, 1.35, 1.36, 1.32, 1.28, 1.25,
                                             1.22, 1.18, 1.15, 1.13, 1.11, 1.07, 1.04, 1.09,
                                             1.10, 1.11, 1.15, 1.19, 1.20, 1.24, 1.28, 1.30,
                                             1.32, 1.32, 1.33, 1.35, 1.35, 1.34, 1.33, 1.31,
                                             1.78, 1.23, 1.20, 1.32, 1.18, 1.34, 1.39, 1.34,
                                             1.21, 1.25, 1.07, 1.13, 1.57, 1.42, 1.36, 1.60,
                                             1.54, 1.60, 1.62, 1.30, 1.39]

    total_anthropogenic_emissions=[]
    for i in range(len(fossil_fuel_emissions)):
        total_anthropogenic_emissions.append(fossil_fuel_emissions[i]+land_use_change_emissions[i])

    return total_anthropogenic_emissions

def main():

    initialAtmosCarbon=315.97 # in ppm
    years=56
    #anthropogenic_emissions=7 # in Gt



    total_anthropogenic_emissions=calc_total_anthropogenic_emissions()
    
    co2=box_model(initialAtmosCarbon,years,total_anthropogenic_emissions)
    plot_carbon(co2)

   
    
if __name__=="__main__":
    main()
