# predator prey model
#Lotka–Volterra equations

import matplotlib.pyplot as plt

def lotkaVolterra(alpha,beta,gamma,delta,x,y):
    dxdt = alpha * x - beta * x * y
    dydt = delta * x * y - gamma * y
    return [dxdt,dydt]

def euler(x,d,dt):
    return x+d*dt

def RK4(f,u,dt):
    k1=f(u)
    u1=u + (dt/2.0) *k1
    k2 = f(u1)
    u2 = u + (dt/2.)*k2
    k3 = f(u2)
    u3 = u + dt*k3
    k4 = f(u3)
    return u + (dt/6.)*(k1 + 2.*k2 + 2.*k3 + k4)

def plot(title,xlbl,ylbl,filename,x,y):
       
    plt.title(title)
    plt.xlabel(xlbl) 
    plt.ylabel(ylbl) 
    plt.xticks()
    plt.plot(x)
    plt.plot(y)
    plt.show()
    #fig = plt.gcf() 
    #fig.savefig(filename+'.png', format='png')
    #plt.close(fig)

def plot2(title,xlbl,ylbl,filename,x,y):
       
    plt.title(title)
    plt.xlabel(xlbl) 
    plt.ylabel(ylbl) 
    plt.xticks()
    plt.plot(x,y)
    plt.show()
    #fig = plt.gcf() 
    #fig.savefig(filename+'.png', format='png')
    #plt.close(fig)

def predatorPrey():

    x0=10.0
    y0=2.0
    alpha=1.0
    beta=1.2
    gamma=4.0
    delta=1.0

    T=15
    dt=0.01

    x=[]
    y=[]
    x.append(x0)
    y.append(y0)

    for i in range(int(T/dt)+1):
        d=lotkaVolterra(alpha,beta,gamma,delta,x[i],y[i])
        x.append(euler(x[i],d[0],dt))
        y.append(euler(y[i],d[1],dt))
        #print(dx,dy,x[i],y[i])

    plot("Predator Prey","Time","Numbers","predatorPrey",x,y)
    plot2("Predator Prey","Prey","Predator","predatorPrey2",x,y)
    


predatorPrey()
